/****************************************
 * Gestion de l'interface avec le shell
 * init_pipe() dmarre le shell et cre les deux pipes
 * envoieshell() envoie une commande  sh (sans '\n' au bout)
 * recoitshell() reoit une ligne de la rponse (ou NULL aprs la dernire)
 * execshell() excute une commande shell sans attendre la rponse
 * erreur() provoque une erreur gnrale
 * *************************************/

#include "prehtml.h"

/* fichiers inclus uniquement dans cette partie, pour cette raison
 * ils ne sont pas dans prehtml.h */
#include <signal.h>
#include <wait.h>

FILE *pin=NULL,*pout=NULL;
static int result_fork;
static char envoyee[LONGUEUR_LIGNE];

void erreur(char *format, ...){
  va_list ap;
  va_start(ap,format);
  dit_ou();
  fprintf(stderr,"prehtml: ");
  vfprintf(stderr,format,ap);
  putc('\n',stderr);
  exit(1);
}

void warning(char *format, ...){
  va_list ap;
  va_start(ap,format);
  dit_ou();
  fprintf(stderr,"prehtml: attention: ");
  vfprintf(stderr,format,ap);
  putc('\n',stderr);
}

void alarme(int numsignal){
  printf("alarme():\n");
  erreur("shell: la commande:\n     \"%s\"\n"
	 " ne rpond pas: peut-tre ceci vient-il de la balise:\n"
	 "     <%s %s>\n"
	 "Vrifiez les guillemets incomplets, crochets, etc...",
	 envoyee,fonction,chaine);
  exit(1);
}

void init_pipe(void){
  char nompipein[]="/tmp/preHTMLinXXXXXX",
    nompipeout[]="/tmp/preHTMLoutXXXXXX";
  char commande[120];
  mktemp(nompipein);mktemp(nompipeout);
  if (mkfifo(nompipein,0666|O_CREAT)==-1 || 
      mkfifo(nompipeout,0666|O_CREAT)==-1)
    erreur("Pas pu crer les fichiers temporaires FIFO");
  result_fork=fork();
  if (result_fork==-1) erreur("Le fork n'a pas march");
  if (result_fork==0){ // process fils
    /* attention  l'ordre des 2 fichiers qui suivent... */
    freopen(nompipeout,"w",stdout);
    freopen(nompipein,"r",stdin);
    snprintf(commande,120,"exec sh %s >%s",nompipein,nompipeout);
    execlp("sh","sh","-c",commande,NULL);
    erreur("exec sh n'a pas march");
  }
  else{
    /* ... car si on ne les ouvre pas dans le mme ordre ici, a bloque */
    pin=fopen(nompipeout,"rb");
    pout=fopen(nompipein,"wb");
    // on vrifie que a fonctionne
    fputs("echo\n",pout);
    fflush(pout);
    while(fgetc(pin)!='\n');
    // puis on efface les fichiers FIFO (mais les pipes restent)
    unlink(nompipein);
    unlink(nompipeout);
  }
  signal (SIGALRM,alarme);
}

void fin_pipe(void){
  if (!pout) erreur("fin_pipe(): pout dj ferm");
  fclose(pout);
  if (!pin) erreur("fin_pipe(): pin dj ferm");
  fclose(pin);
  pout=pin=NULL;
  kill(result_fork,SIGTERM);
  wait(NULL);
}
  

void envoieshell(char *s){  // envoie une commande  sh
  char s2[LONGUEUR_LIGNE];
  if (!pout) erreur("envoieshell: pipe out non ouvert!");
  memcpy(envoyee,s,LONGUEUR_LIGNE);
  fprintf(pout,"echo COMMANDE_TEST;%s\necho %s\n",s,CODE_FIN_SHELL);
  fflush(pout);
  alarm(3); // alarme dans 3 secondes !
  fgets(s2,LONGUEUR_LIGNE,pin);
  alarm(0); // OK, j'ai reu quelques chose
  if (strcmp(s2,"COMMANDE_TEST\n"))
    erreur("shell: Avant mme d'envoyer \"%s\",\n"
	   "  je reois \"%s\". Erreur interne ?",
	   s,s2);
}

char *recoitshell(char *s){ // rcupre la rponse de sh et met une ligne
                            // dans *s; renvoie NULL si c'est fini
  int i;
  fgets(s,LONGUEUR_LIGNE,pin);
  i=strlen(s);
  if (i) if (s[i-1]=='\n') s[i-1]='\0';
  if (!strcmp(s,CODE_FIN_SHELL)) return NULL;
  return s;
}

void execshell(char *s){ // envoie une commande  sh mais nglige la rponse
  char bidon[LONGUEUR_LIGNE];
  envoieshell(s);
  while (recoitshell(bidon));
}
